/* 
 * Background Saver - BeOS Screensaver module
 * Blanks the screen and displays the current workspace background image
 *
 * Written by and copyright © 2000 by John Yanarella (yanarejm@muohio.edu)
 *
 * version 1.0 (April 3rd, 2000)
 *    - original release
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 *
 */
 
#include <Screen.h>
#include <TextView.h>
#include <File.h>
#include <Node.h>
#include <NodeInfo.h>

#include "BackgroundSaver.h"
#include "background.h"

static const char* TITLE =
	"Background Saver v1.0\n";
	
static const char* CREDITS = 
	"Desktop Background Screensaver\n\n"
	"Blanks the screen and displays the desktop\n"
	"background for the current workspace.\n\n"
	"Written by and copyright © 2000\n"
	"John Yanarella (yanarejm@muohio.edu)\n";

extern "C" _EXPORT BScreenSaver *instantiate_screen_saver(BMessage *message, image_id image)
{
	return new BackgroundSaver(message, image);
}

BackgroundSaver::BackgroundSaver(BMessage *msg, image_id image)
 : BScreenSaver(msg, image)
{
	m_preview = false;
	m_image = image;
	m_icon = NULL;
}

BackgroundSaver::~BackgroundSaver() 
{
}

void BackgroundSaver::StartConfig(BView *view)
{
	BTextView* creditsView = new BTextView(view->Bounds(), "credits", view->Bounds().InsetBySelf(5.0,0.0), B_FOLLOW_NONE, B_WILL_DRAW);
	
	creditsView->MakeEditable(false);
	creditsView->SetStylable(true);
	creditsView->MakeSelectable(false);	
	creditsView->SetAlignment(B_ALIGN_LEFT);
	creditsView->SetViewColor(216,216,216,0);
	
	BFont font(be_bold_font);
	font.SetSize(12.0);
	creditsView->SetFontAndColor(&font);
	creditsView->Insert(TITLE,strlen(TITLE));
	
	BFont font2(be_plain_font);
	font2.SetSize(10.0);
	creditsView->SetFontAndColor(&font2);	
	creditsView->Insert(CREDITS, strlen(CREDITS));
		
	view->AddChild(creditsView);
}


status_t BackgroundSaver::StartSaver(BView * view, bool preview)
{
	if (preview) // grab this screensaver's icon
	{
		image_info info;
		BFile addon_file;
		if ((get_image_info(m_image, &info) == B_OK ) &&
			(addon_file.SetTo(info.name, O_RDONLY ) == B_OK))
		{
			BNodeInfo addon_node_info((BNode*)&addon_file);
			m_icon = new BBitmap(BRect(0,0,31,31), B_CMAP8);
			if (addon_node_info.GetTrackerIcon(m_icon, B_LARGE_ICON) == B_OK)
			{
				SetTickSize(1000000);
				m_preview = true;
				return B_OK;
			} 
			else 
			{
				delete m_icon;
				m_icon = NULL;
				return B_ERROR;
			}
		} 
		else 
		{
			delete m_icon;
			m_icon = NULL;
			return B_ERROR;
		}
		return B_OK;
	}
	
	BScreen screen;
	view->SetViewColor(screen.DesktopColor());
	
	Mode m_mode;
	BPoint m_offset;
	BBitmap* bitmap = GetCurrentBackgroundImage(m_mode, m_offset);
	SetBackgroundImage(view, bitmap, m_mode, m_offset);	
	delete bitmap;
	
	return B_OK;
}

void BackgroundSaver::StopSaver() 
{
	if (m_icon != NULL)
		delete m_icon;	
}

void BackgroundSaver::Draw(BView *view, int32 frame)
{
	// preview draw
	if (m_preview) {
		if (frame == 0) {
			rgb_color color = { 216, 216, 216, 0 };
			view->SetHighColor(color);
			view->FillRect(view->Bounds());
			view->SetDrawingMode(B_OP_OVER);
			BPoint pt(view->Bounds().Width()/2 - 16, view->Bounds().Height()/2 - 16);
			view->DrawBitmap(m_icon, pt);
		}		
		return;
	}
}